import std/[httpclient, json, logging, smtp, strutils, times]
import docopt, zero_functional
import cliutils/[queue_logger, config]
import stock_watcherpkg/cliconstants

type
  Config = object
    cfg: CombinedConfig
    apiKey: string
    apiRoot: string
    notifyAddresses: seq[string]
    smtpHostname: string
    smtpPassword: string
    smtpPort: int
    smtpUsername: string

proc sendMessages(
    exitStatus: int,
    cfg: Config,
    messages: seq[QueuedLogMessage],
    dryRun = false
  ) {.raises: [].} =

  try:
    var subjStatus: string
    if dryRun:
      subjStatus = "dry run"
    elif exitStatus == QuitSuccess:
      subjStatus = "success"
    else:
      subjStatus = "FAILED"

    let dateStr = now().format("yyyy-MM-dd")
    let msg = createMessage(
      "Stock Watcher ($#) -- $#" % [subjStatus, dateStr],
      MESSAGE_TEMPLATE % [ messages.join("\p") ],
      cfg.notifyAddresses)

    let smtpConn = newSmtp(useSsl = true)
    smtpConn.connect(cfg.smtpHostname, Port cfg.smtpPort)
    smtpConn.auth(cfg.smtpUsername, cfg.smtpPassword)
    smtpConn.sendmail("operations@probatem.com", cfg.notifyAddresses, $msg)

  except CatchableError:
    try:
      let ex = getCurrentException()
      warn "Failed to send notification email: " & ex.msg
      debug ex.getStackTrace
    except CatchableError: discard

proc loadConfig(args: Table[string, Value]): Config =
  let filepath = findConfigFile("stock_watcher")

  let cfg = initCombinedConfig(filepath, args)

  result = Config(
    cfg: cfg,
    apiKey: cfg.getVal("api-key"),
    apiRoot: cfg.getVal("api-root"),
    notifyAddresses: cfg.getJson("notify-addresses").getElems --> map(it.getStr),
    smtpHostname: cfg.getVal("smtp-hostname"),
    smtpPassword: cfg.getVal("smtp-password"),
    smtpPort: parseInt(cfg.getVal("smtp-port")),
    smtpUsername: cfg.getVal("smtp-username"))

when isMainModule:
  var exitStatus = QuitSuccess
  var cfg: Config
  let args = docopt(USAGE, version = VERSION)

  let queuedLogger = newQueueLogger(
    levelThreshold = lvlInfo,
    fmtStr = "$levelname -")

  let consoleLogger = newConsoleLogger(
    levelThreshold = lvlInfo,
    fmtStr = "stock_watcher - $levelname: ")

  try:
    if args["--debug"]:
      queuedLogger.levelThreshold = lvlDebug
      consoleLogger.levelThreshold = lvlDebug

    if args["--help"]:
      stderr.writeLine(USAGE)
      quit()

    logging.addHandler(consoleLogger)
    logging.addHandler(queuedLogger)

    let cfg = loadConfig(args)
    let http = newHttpClient()
    http.headers = newHttpHeaders({
      "Content-Type": "application/json",
      "Authorization": "Bearer " & cfg.apiKey
    })

  except CatchableError:
    let ex = getCurrentException()
    fatal ex.msg
    debug ex.getStackTrace
    exitStatus = QuitFailure

  finally:
    sendMessages(exitStatus, cfg, queuedLogger.messages, args["--dry-run"])
    quit(exitStatus)
